 

import java.io.*;
import java.util.*;

/**
* Implements an object with this same type which represents a
* deterministic finite automaton or FA. A FA is defined by the
* following parameters: a set of states, an input alphabet, a starting
* state, a transition function, and a set of accepting states. This
* class implements all three of these parameters, although the starting
* state and the set of accepting states are ommited and instead merged
* into the set of states. This set is implemented using a {@link #States
* list} of objects of type {@link State}. The starting state is located
* at the beginning of this list, and an accepting state in the set is
* indicated through the state's {@link State#acceptFlag accepting flag}.
* The transition function is also implemented with a {@link #Transitions
* list} of objects of type {@link Transition}. The input alphabet is
* represented through a {@link #Alphabet list} of {@link Character}
* objects, each representing a specific character in the input alphabet.
*
* @see State
* @see Transition
*/

public class FA implements Serializable 
{
 //~ Instance variables 

 /** The FA's identifying name. */
 private String FA_Name;

 /**
 * The list of {@link Character} objects reprenting the input alphabet
 * of the FA. A {@link Vector} is used to implement the list to
 * allow virtually an unlimited length for the input alphabet.
 */
 private Vector Alphabet = new Vector();
 
 /**
 * The list of {@link State} object representing the set of states in
 * the FA, implemented as a {@link Vector} to allow virtually for an
 * unlimited number of states. The starting state is the first
 * element of the list, while accepting states are implicitly
 * indicated within the {@link State} objects representing the states
 * in the list.
 */
 private Vector States = new Vector();

 /**
 * The list of {@link Transition} object representing the transition
 * function of the FA, and implemented as a {@link Vector} to allow
 * virtually for an unlimited number of transitions.
 */
 private Vector Transitions = new Vector();

 //~ Constructors 
 /**
 * The default constructor for this class. A default {@link
 * #FA_Name} is provided for the FA object when calling this
 * constructor.
 */

 public FA() 
 {
   setName("FA_1");
 }
 
 /**
 * The class constructor where the FA name is indicated.
 *
 * @param name the {@link #FA_Name FA name}
 */
 public FA(String name) 
 {
  setName(name);
 }
 
 //~ Methods 
 /**
 * Sets the input alphabet of the FA. This method accepts a regular
 * array of characters as parameter and copies the contents into
 * {@link #Alphabet}. It also checks if any input in the previous
 * input alphabet no longer exists in the new input alphabet, and any
 * transition on all of these inputs are removed from the FA as well
 * using the {@link #deleteAllTransitionWithInput(char)} method.
 *
 * @param alphabet the array of characters representing the input
 * alphabet
 *
 * @see FAChecker#charArraytoVector(char[])
 */

 public void setAlphabet(char[] alphabet) 
 {
 /* keep a copy of the existing alphabet, for monitoring
 * removed input characters.
 */
 Vector v = (Vector) Alphabet.clone();
 Character c;
 /* clear the current alphabet first */
 Alphabet.removeAllElements();
 /* copy the contents of the alphabet array to Alphabet vector.
 */
 Alphabet = FAChecker.charArraytoVector(alphabet);
 /* remove from the previous alphabet any input that still
 * exists in the new alphabet. the remaining inputs are the
 * were erased from the alphabet.
 */
 v.removeAll(Alphabet);
 /* delete any transition on all removed inputs. */
  for (int i = 0; i < v.size(); i++) 
  {
   c = (Character) v.elementAt(i);
   deleteAllTransitionWithInput(c.charValue());
  }
 }

 /**
 * Returns the input alphabet in {@link #Alphabet} as an array of
 * characters.
 *
 * @return the input alphabet
 *
 * @see FAChecker#vectortoCharArray(Vector)
 */

 public char[] getAlphabet() 
 {
  char[] get_alphabet = FAChecker.vectortoCharArray(this.Alphabet);
  if(get_alphabet[0] == '')
  {
   char[] mod_alphabet = new char[get_alphabet.length - 1];
   for(int i = 0; i<get_alphabet.length-1; i++)
   {
   	mod_alphabet[i] = get_alphabet[i+1];
   }
   return mod_alphabet;
  }
  return get_alphabet;  
 }

 /**
 * Sets the FA name.
 *
 * @param name the FA name
 */
 
 public void setName(String name) 
 {
  this.FA_Name = name;
 }

 /**
 * Returns the FA name.
 *
 * @return {@link #FA_Name this.FA_Name}
 */

 public String getName() 
 {
  return this.FA_Name;
 }

 /**
 * Returns the next available transition index for a transition from
 * the source to the destination state on any input.
 *
 * @param source the source state
 * @param destination the destination state
 *
 * @return the transition index
 */

 public int getNextIndex(State source, State destination) 
 {
  int index = 0;
  Transition t;
  /* return a 0 for the first transition. */
  if (getTransitionCount() == 0) 
  {
   return index;
  }
  /* for every transition on the same path, increment the index.
  */
  for (int i = 0; i < getTransitionCount(); i++) 
  {
   t = getTransitionAt(i);
   if ((t.getSource() == source) && (t.getDestination() == destination)) 
   {
    index++;
   }
  }
  return index;
 }
 
 /**
 * Sets the starting state of the FA to the state located in the
 * specified position in the state list.
 *
 * @param position the position of the state to be set as starting
 * state in the state list.
 *
 * @return true if the state position is valid; false otherwise
 *
 * @see #setStartingState(State)
 */

 public boolean setStartingState(int position) 
 {
  if (!isValidStatePosition(position)) 
  {
   return false;
  }
  States.addElement(States.elementAt(0));
  States.setElementAt(States.elementAt(position), 0);
  States.setElementAt(States.elementAt(States.size() - 1), position);
  States.removeElementAt(States.size() - 1);
  return true;
 }

 /**
 * Sets the starting state of the FA to the specified state.
 *
 * @param state the state to be set as starting state
 *
 * @return true is state is in the state list; false otherwise
 *
 * @see #setStartingState(int)
 */

 public boolean setStartingState(State state) 
 {
  if (!States.contains(state)) 
  {
   return false;
  }
  return setStartingState(States.indexOf(state));
 }

 /**
 * Returns the current starting state of the FA.
 *
 * @return the first element in the state list
 */
 public State getStartingState() 
 {
  return getStateAt(0);
 }

 /**
 * Returns the state located in the given position in the state list. 
 *
 * @param position the position of the state in the state list
 *
 * @return the state at the given position in the state list if 0 if
 * the state position is valid; null otherwise
 */

 public State getStateAt(int position) 
 {
  if (!isValidStatePosition(position)) 
  {
   return null;
  }
  return (State) States.elementAt(position);
 }

 /**
 * Returns the number of states in the state list.
 *
 * @return the size of {@link #States}
 */

 public int getStateCount() 
 {
  return States.size();
 }

 /**
 * Returns the position (or index) of the state in the state list.
 *
 * @param state the state to be located in the state list
 *
 * @return the position of state in the state list.
 */

 public int getStateIndex(State state) 
 {
  return States.indexOf(state);
 }
 
 /**
 * Returns the list of states in the FA (converted to {@link State}
 * array).
 *
 * @return the array of states in the FA
 */

 public State[] getStateList() 
 {
  if (States.isEmpty()) 
  {
   return null;
  }
  /* copy the States vector to a State array */
  State[] statelist = new State[States.size()];
  States.copyInto(statelist);
  return statelist;
 }

/**
* Returns the transition with the given source state and input.
*
* @param currState the source state of the transition
* @param input the transition input
*
* @return the transition from the given source state on the given
* input
*/

 public Transition getTransition(State currState, char input) 
 {
  for (int i = 0; i < getTransitionCount(); i++) 
  {
   Transition t = getTransitionAt(i);
   if ((t.getSource() == currState) && (t.getInput() == input)) 
   {
    return t;
   }
  }
  return null;
 }
 
 /**
 * Returns the transition located in the specified position in the
 * transition list.
 *
 * @param position the position of the transition
 *
 * @return the transition at the specified position
 */

 public Transition getTransitionAt(int position) 
 {
  if (!isValidTransitionPosition(position)) 
  {
   return null;
  }
  return (Transition) Transitions.elementAt(position);
 }

 /**
 * Returns the number of transitions in the transition list.
 *
 * @return size of {@link #Transitions}
 */

 public int getTransitionCount() 
 {
  return Transitions.size();
 }

 /**
 * Returns the position of the transition in the transition list.
 *
 * @param transition the position of the transition
 *
 * @return the transition to be located
 */

 public int getTransitionIndex(Transition transition) 
 {
  return Transitions.indexOf(transition);
 }
 
 /**
 * Returns the list of transitions in the FA (converted to an array
 * of {@link Transition Transitions}.
 *
 * @return the transition array
 */

 public Transition[] getTransitionList() 
 {
  if (Transitions.isEmpty()) 
  {
   return null;
  }
  Transition[] tlist = new Transition[Transitions.size()];
  Transitions.copyInto(tlist);
  return tlist;
 }

 /**
 * Checks if a state position is within available range in the state
 * list.
 *
 * @param position the state position
 *
 * @return true if position is valid; false otherwise
 */

 public boolean isValidStatePosition(int position) 
 {
  if ((position < 0) || (position > (States.size() - 1))) 
  {
   return false;
  }
  return true;
 }
 
 /**
 * Checks if a transition position is within available range in the
 * transition list.
 *
 * @param position the transition position
 *
 * @return true if position is valid; false otherwise
 */
 
 public boolean isValidTransitionPosition(int position) 
 {
  if ((position < 0) || (position > (Transitions.size() - 1))) 
  {
   return false;
  }
  return true;
 }

 /**
 * Adds a state to the end of the state list. The state count is
 * automatically incremented.
 *
 * @param state the state to be added.
 *
 * @return true if state has been added successfully; false if the
 * state list already contains the same state or if the state
 * is null
 */

 public boolean addState(State state) 
 {
  if (States.contains(state) || (state == null)) 
  {
   return false;
  }
 States.addElement(state);
 return true;
 }

 /**
 * Adds a transition to the end of the transition list. The transition
 * count is automatically incremented.
 *
 * @param t the transition to be added
 *
 * @return true if the transition has been added; false if a similar
 * transition is already in the list or if the given
 * transition is null
 */

 public boolean addTransition(Transition t) 
 {
  /* return if a similar transition already exists. */
  if (Transitions.contains(t) || (t == null) || !States.contains(t.getSource()) || !States.contains(t.getDestination()) || !Alphabet.contains(new Character(t.getInput()))) 
  {
   return false;
  }
  t.setIndex(this.getNextIndex(t.getSource(), t.getDestination()));
  Transitions.addElement(t);
  return true;
 }

 /**
 * Deletes all transitions that are using the given input as
 * transition input.
 *
 * @param input the transition input
 */
 
 public void deleteAllTransitionWithInput(char input) 
 {
  Vector v = new Vector();
  for (int i = 0; i < getTransitionCount(); i++) 
  {
   Transition t = getTransitionAt(i);
   if (t.getInput() == input) 
   {
    v.addElement(t);
   }
  }
  for (int i = 0; i < v.size(); i++) 
  {
   deleteTransition((Transition) v.elementAt(i));
  }
 }
 
 /**
 * Delete the state in the given position in the state list. The state
 * count is automatically decremented, and all transitions using this
 * state as source, destination, or both are deleted as well.
 *
 * @param position the state's position in the state list
 *
 * @return true on successfull removal; false if position is invalid
 */

 public boolean deleteState(int position) 
 {
  if (!isValidStatePosition(position)) 
  {
   return false;
  }
  State state = getStateAt(position);
  /* delete all transitions using this state as either source
  * state or destination state.
  */
  for (int i = 0; i < Transitions.size(); i++) 
  {
   if ((getTransitionAt(i).getSource() == state)
   || (getTransitionAt(i).getDestination() == state)) 
   {
    deleteTransition(getTransitionAt(i));
    i--;
   }
  }
  /* now delete the state from the statelist */
  States.removeElementAt(position);
  return true;
 }

 /**
 * Delete the state from the state list. The state count is
 * automatically decremented. The state count is automatically
 * decremented, and all transitions using this state as source,
 * destination, or both are deleted as well.
 *
 * @param state the state to be deleted
 *
 * @return true on sucessful removal; false if state is not in state
 * list
 */
 
 public boolean deleteState(State state) 
 {
  if (!States.contains(state)) 
  {
   return false;
  }
  return deleteState(States.indexOf(state));
 }

 /**
 * Delete the transition in the given position from the transition
 * list. The transition count is automatically decremented.
 *
 * @param position the position of the transition in the list
 *
 * @return true on success; false if position is invalid
 */

 public boolean deleteTransition(int position) 
 {
  if (!isValidTransitionPosition(position)) 
  {
   return false;
  }
  Transition a = getTransitionAt(position);
  for (int i = 0; i < getTransitionCount(); i++)
  {
   Transition t = getTransitionAt(i);
   if ((t.getSource() == a.getSource())
   && (t.getDestination() == a.getDestination())
   && (t.getIndex() > a.getIndex())) 
   {
    t.setIndex(t.getIndex() - 1);
   }
  }
  Transitions.removeElementAt(position);
  return true;
 }

 /**
 * Delete the transition from the transition list. The transition
 * count is automatically decremented.
 *
 * @param transition the transition to be deleted
 *
 * @return true on success; false if transition is not in transition
 * list
 */
 
 public boolean deleteTransition(Transition transition) 
 {
  if (!Transitions.contains(transition)) 
  {
   return false;
  }
  return deleteTransition(Transitions.indexOf(transition));
 }
}
