 

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
* Provides a set of methods for checking the FA and its elements. This
* class also contains a set of error messages, warning messages, and
* other messages used in error-checking the FA along with a conversion
* method for vectors to character arrays and vice-versa.
*/

public class FAChecker 
{
 //~ Static variables/initializers
 
 /** Constant for an OK result from a warning message dialog box. */
 public static int OK = JOptionPane.OK_OPTION;
  
 /** Constant for a CANCEL result from a warning message dialog box. */
 public static int CANCEL = JOptionPane.CANCEL_OPTION;

 /**
 * Zero-argument constant for use calling methods requiring string
 * array parameters.
 */
 public static String[] NO_ARGS = new String[0];

 /**
 * The array of messages for possible errors in the FAEditor or the
 * FA.
 */

 private static final String[][] errorMsg = 
 {
  {
   "Invalid file. \nCannot open ",
   "Invalid file. \nCannot save to ", "Cannot import file.\n"
  }, 
  {
   "Invalid file. \nCannot export FA to ",
   "Invalid FA.\nFA is still incomplete.\n",
   "* State <S> is missing a transition on input <I>.",
   "* FA has no accepting state."
  }, 
  {
   "No FA available. \nCreate a Finite Automaton first.", "Invalid FA name.",
   "Invalid FA input alphabet.", "Invalid input string."
  }, 
  {
   "No state selected. \nSelect a state first.",
   "Invalid state name.",
   "State name is already in use by another state.",
   "Cannot delete starting state.\n"
   + "Change starting state first before deleting this state."
  }, 
  {
   "No transition selected. \nSelect a transition first.",
   "Invalid transition input.",
   "A transition from this state on the given input already exists."
  },
   {
   "Exporting eNFA and FA not yet allowed."
  }
 };

 /** Array of miscellaneous messages. */
 private static final String[] infoMsg = 
 {
  "Graphical Finite Automata Editor\nDevelopers: \n" + "Jayson Vincent C. Angeles" 
  + "\nRichelle K. Arjona" + "\n\n\n\nGraphical DFA Editor\nDevelopers: \n" 
    + "\nJulie Ann Abueva\n" + "Galahad Gamo Jr.\n" + "Catherine Oscares\n"
  + "Edsel Conrad Sabulao\n" + "Evan Vidal Sagge\n"
  + "Christine Suamen", "FA successfully saved to ",
  "FA successfully exported to ","Epsilon transitions can be added\nby inputting '\\e' or ''" 
 };

 /** Array of warning messages. */
 private static final String[] warningMsg = 
 {
  "Are you sure you want to create a new FA? \n"
  + "Creating a new FA will discard all unsaved changes \n"
  + "to the current FA.",
  "Are you sure you want to open another FA file? \n"
  + "Opening a FA file will discard all unsaved changes \n"
  + "to the current FA.",
  "Are you sure you want to delete this state? \n"
  + "All transitions to or from this state will also be deleted.",
  "Are you sure you want to delete this transition?",
  "Are you sure you want to change the input alphabet? \n"
  + "All transitions on inputs which were removed \n"
  + "will also be deleted.",
  "The FA has no non-accepting state. \n"
  + "Are you sure this FA is complete?",
  "A FA still exists in the workspace panel. \n"
  + "Do you want to save this FA to disk?",
  "Do you want to save the current FA before closing it?",
  "The FA has no accepting state. \n"
  + "Are you sure this is complete?",
 };

 //~ Constructors 
 /**
 * The sole constructor for this class
 */
 public FAChecker() 
 {
 }

 //~ Methods 
 /**
 * Checks if all states in the fa are accepting states.
 *
 * @param parent the parent component for the message dialog boxes.
 * @param fa the fa for checking
 *
 * @return true if all states are accepting; false if at least one
 * state is non-accepting
 */

 public static boolean isAllAcceptingStates(Component parent, FA fa) 
 {
  State[] statelist = fa.getStateList();
  for (int i = 0; i < statelist.length; i++) 
  {
   State state = statelist[i];
   if (!state.getAcceptFlag()) 
   {
    return false;
   }
  }
  return true;
 }

 public static boolean acceptingStateExists(FA fa) 
 {
  State[] statelist = fa.getStateList();
  for (int i = 0; i < statelist.length; i++) 
  {
   State state = statelist[i];
   if (state.getAcceptFlag()) 
   {
    return true;
   }
  }
  
  return false;
 }
 /**
 * Checks if a character should require a preceding backslash as used
 * in FA specification format.
 *
 * @param c the character to be checked
 *
 * @return true if character requires backslash; false otherwise
 */
 public static boolean isBackslashCharacter(char c) 
 {
  if ((c == ' ') || (c == ']') || (c == '[') || (c == '|')
  || (c == ';') || (c == ':') || (c == '{') || (c == '}')
  || (c == '\\') || (c == '"')) 
  {
   return true;
  }
  return false;
 }

 /**
 * Checks if a FA is complete by checking if the the transition list
 * (or transition function) is complete and an accepting state is
 * present. If the FA does not comply with both of these conditions,
 * then it is incomplete. An error message if displayed if it is
 * incomplete. NOTE: A FA with no non-accepting state (i.e. all
 * states are accepting) is still complete if the above conditions
 * are met, although inefficient and thus useless. In such a case, a
 * warning message is displayed instead of an error message.
 *
 * @param parent the parent component for the message dialog boxes.
 * @param fa the fa for checking
 *
 * @return true if fa is complete; false otherwise
 */

 public static boolean isFAComplete(Component parent, FA fa) 
 {
  String transcheckmsg = getIncompleteTransitions(fa);
  /* if there are incomplete transitions and an accepting state
  * is available, then fa is complete.
  */
  if ((transcheckmsg.length() == 0)) 
  {
   /* warn if a fa only has accepting states. */
   if (isAllAcceptingStates(parent, fa)) 
   {
    if (displayWarning(parent, 106) == OK) 
    {
     return true;
    } 
    else 
    {
     return false;
    }
   }
   /* warn if there's no accepting state. */
   if (!acceptingStateExists(fa)) 
   {
    if (displayWarning(parent, 109) == OK) 
    {
     return true;
    } 
    else 
    {
     return false;
    }
   }
   return true;
  }
  /* display the error message if fa is incomplete. */
  JOptionPane.showMessageDialog(parent,  getErrorMsg(202, NO_ARGS)  + transcheckmsg,  "Error 202",  JOptionPane.ERROR_MESSAGE);
  return false;
 }
 
 /**
 * Shows a information message box that tells the user that the \
 * finite automaton is not a DFA
 *
 * Called whenever the simulation button is called and the
 * FA is not a DFA
 */
 
 public static void NotADFA(Component parent)
 {
 	JOptionPane.showMessageDialog(parent,"NFA and eNFA simulation not allowed\n","Error 303",JOptionPane.ERROR_MESSAGE);
 }
 
 /**
 * Checks if the fa is empty, and displays an error message if it is.
 *
 * @param parent the parent component for the message dialog boxes.
 * @param fa the fa for checking
 * @param showMsg if true, sets to show an error message for an empty
 * fa
 *
 * @return true if fa is empty; false otherwise
 */

 public static boolean isEmptyFA(Component parent, FA fa,
 boolean showMsg) 
 {
  if (fa == null) 
  {
   if (showMsg) 
   {
    displayError(parent, 301);
   }
   return true;
  }
  return false;
 }
 
 /**
 * Returns the error message corresponding with the error code. Some
 * error messages require certain arguments.
 *
 * @param errorCode the error code
 * @param args string arguments
 *
 * @return the string containing the error message
 */

 public static String getErrorMsg(int errorCode, String[] args) 
 {
  String e = Integer.toString(errorCode);
  int i = Integer.valueOf(e.substring(0, 1)).intValue();
  int j = Integer.valueOf(e.substring(2, 3)).intValue();
  String msg = errorMsg[i - 1][j - 1];
  if ((errorCode == 101) || (errorCode == 102) || (errorCode == 201)) 
  {
   msg = msg + args[0];
  }
  return msg;
 }

 /**
 * Checks if a name is an identifier. An identifier must not be of
 * zero-length and must be composed only of alphanumeric characters
 * and the underscore ('_') character.
 *
 * @param s the name to be checked
 *
 * @return true if the name is an identifier; false otherwise
 */

 public static boolean isIdentifier(String s) 
 {
  if ((s == null) || (s.length() == 0)) 
  {
   return false;
  }
  for (int i = 0; i < s.length(); i++) 
  {
   if (!Character.isLetterOrDigit(s.charAt(i))
   && (s.charAt(i) != '_')) 
   {
    return false;
   }
  }
  return true;
 }

 /**
 * Checks if there are incomplete transitions in the FA by checking
 * if exactly one transition exists for every combination of a state
 * in the state list and an input from the input alphabet.
 *
 * @param fa the FA
 *
 * @return a string of zero (0) length if no incomplete transitions in
 * the fa; else a string showing the incomplete transitions
 */

 public static String getIncompleteTransitions(FA fa) 
 {
  State[] states = fa.getStateList();
  char[] alphabet = fa.getAlphabet();
  //JOptionPane.showConfirmDialog(null,alphabet[0]);
  char[][] a = new char[states.length][alphabet.length];
  int count = 0;
  String msg = new String();
  for (int i = 0; i < states.length; i++) 
  {
   for (int j = 0; j < alphabet.length; j++) 
   {
    /* check if transition exists on each combination of
    * state and input.
    */
 
    if (!transitionExists(fa, states[i], alphabet[j])) 
    {
     a[i][j] = alphabet[j];
     count++;
     /* limit to 10 messages */
     if (count < 10) 
     {
      msg += "* A transition from state ";
      msg += states[i].getName();
      msg += " on input ";
      msg += alphabet[j];
      msg += " is missing.\n";
     }  
     else if (count == 10) 
     {
      msg += "[More missing transitions...]\n";
     }
    } 
   }
  }
  return msg;
 }

 /**
 * Returns the message corresponding with the message code. Some
 * messages require certain arguments.
 *
 * @param infoCode the message code
 * @param args string arguments
 *
 * @return the string containing the message
 */

 public static String getInfoMsg(int infoCode, String[] args) 
 {
  String msg = infoMsg[infoCode];
  if ((infoCode == 1) || (infoCode == 2)) 
  {
   msg = msg + args[0];
  }
  return msg;
 }

 /**
 * Checks if a state name is still available and not being used by any
 * existing state in the state list, and displays an error if not.
 *
 * @param parent the parent component for the message dialog boxes.
 * @param fa the fa for checking
 * @param StateName the state name
 * @param state any state to be excluded from checking (should be set
 * to null in most cases)
 *
 * @return true if state name is available; false otherwise
 */

 public static boolean isStateNameAvailable(Component parent, FA fa,
 String StateName, State state) 
 {
  for (int i = 0; i < fa.getStateCount(); i++) 
  {
   if (fa.getStateAt(i).getName().equalsIgnoreCase(StateName)
   && (fa.getStateIndex(state) != i)) 
   {
    displayError(parent, 403);
    return false;
   }
  }
  return true;
 }

 /**
 * Checks if a state is currently selected in the workspace panel.
 * 
 * @param parent the parent component for the message dialog boxes
 * @param w the workspace panel
 *
 * @return true if a state is selected; false otherwise
 */
 
 public static boolean isStateSelected(Component parent, WorkSpacePanel w) 
 {
  if (w.getSelectedState() == null) 
  {
   displayError(parent, 401);
   return false;
  }
  return true;
 }

 /**
 * Checks if a transition is currently selected in the workspace
 * panel.
 *
 * @param parent the parent component for the message dialog boxes
 * @param w the workspace panel
 *
 * @return true if a transition is selected; false otherwise
 */

 public static boolean isTransitionSelected(Component parent,WorkSpacePanel w) 
 {
  if (w.getSelectedTransition() == null) 
  {
   displayError(parent, 501);
   return false;
  }
  return true;
 }

 /**
 * Checks if a FA name is valid.
 *
 * @param parent the parent component for the message dialog boxes
 * @param name the FA name
 *
 * @return true if fa name is valid; false otherwise
 *
 * @see #isIdentifier
 */

 public static boolean isValidFAName(Component parent, String name) 
 {
  if ((name.length() == 0) || (name == null) || !isIdentifier(name)) 
  {
   displayError(parent, 302);
   return false;
  }
  return true;
 }

 /**
 * Checks if an input is valid, and display a message if not.
 *
 * @param parent the parent component for the message dialog boxes
 * @param fa the current FA
 * @param input a string containing the input
 *
 * @return true if input is valid; false otherwise
 */
 public static boolean isValidInput(Component parent, FA fa, String input) 
 {
  /* input should be a single character only, and it should exist
  * in the input alphabet.
  */
  String epsilon = "";
  if( WorkSpacePanel.isItDFA() || WorkSpacePanel.isItNFA() )
  {
   if ((input.length() != 1) || !charArraytoVector(fa.getAlphabet()).contains(new Character(input.charAt(0)))) 
   {
    if (parent != null) 
    {
     displayError(parent, 502);
    }
    return false;
   }	
  } 
   
  if( WorkSpacePanel.isIteNFA() || WorkSpacePanel.isItFA() )
  {
   if(input.charAt(0)==92 && input.charAt(1)=='e')
   {   	
   	return true;   	
   } 
   else if((input.charAt(0) == FAChecker.epsilonChar() ) )
   {
   	return true;
   }    
   else if ((input.length() != 1) || !charArraytoVector(fa.getAlphabet()).contains(new Character(input.charAt(0)))) 
   {
    if (parent != null) 
    {
    
     displayError(parent, 502);
    }    
    return false;
   }	
  }  
  return true;
 }

 /**
 * Checks if the input alphabet string is valid, and display a message
 * if not. An input alphabet in its string representation can use
 * ranging, so the alphabet is checked when it is expanded into a
 * character array.
 *
 * @param parent the parent component for the message dialog boxes
 * @param alphabet the input alphabet (string representation)
 *
 * @return true if alphabet is valid; false otherwise
 */

 public static boolean isValidInputAlphabet(Component parent,String alphabet) 
 {
  /* alphabet should not be empty */
  /* check also if it is valid when expanded */
  /* the input alphabet must include an e when the FA is eNFA or FA*/  
  if ((alphabet.length() == 0) || (alphabet == null)  || (expandAlphabetString(alphabet) == null))  
  {
   displayError(parent, 303);
   return false;
  }    
  return true;
 } 
 
 /**
  *checks whether the input alphabet is alphanumeric for DFA and NFA 
  *and the input alphabet for eNFA and FA includes the epsilon. If 
  *this is satisfied, a true is returned else a false. Also checks 
  *for a dash which symbolizes for a range
  *
  *@param parent the parent component for the message dialog boxes
  *@param alphabet the input alphabet
  *
  *@return true if input alphabet is alphanumeric with epsilon transition
  **/
 public static boolean isValidInputAlphabetforAllTypes(Component parent,String alphabet)
 {
  boolean isIt = true;
  if( WorkSpacePanel.isItDFA() || WorkSpacePanel.isItNFA() || WorkSpacePanel.isIteNFA() || WorkSpacePanel.isItFA())
  {   
   for(int i = 0;i<alphabet.length();i++)
   {   	
    if(alphabet.charAt(i)>=32 && alphabet.charAt(i)<=126 )	
  	{  	 
  	 isIt = true; 
  	}
  	else
  	{
  	 displayError(parent, 303);  	 
  	 return false;
  	}
   }
   return isIt;
  }
  displayError(parent, 303);  	 
  return false;
 }
 
 
 /**
 * Checks if an input string is valid and the inputs in the string are
 * within the input alphabet, and displays a message if not. Empty
 * strings are valid input strings.
 *
 * @param parent the parent component for the message dialog boxes
 * @param fa the current FA
 * @param input a string containing the input
 *
 * @return true if input is valid; false otherwise
 */
 
 public static boolean isValidInputString(Component parent, FA fa, String input) 
 {
  /* empty string, return valid */
  if ((input == null) || (input.length() == 0)) 
  {
   return true;
  }
  /* check if each input in the string is valid */
  for (int i = 0; i < input.length(); i++) 
  {
   if (!isValidInput(null, fa, String.valueOf(input.charAt(i)))) 
   {
    displayError(parent, 304);
    return false;
   }
  }
  return true;
 } 

 /**
 * Checks if a state name is valid, and displays a message if not.
 *
 * @param parent the parent component for the message dialog boxes
 * @param name the state name
 *
 * @return true if name is valid; false otherwise
 *
 * @see #isIdentifier
 */

 public static boolean isValidStateName(Component parent, String name) 
 {
  if ((name.length() == 0) || (name == null) || !isIdentifier(name)) 
  {
   displayError(parent, 402);
   return false;
  }
  return true;
 }

 /**
 * Checks if a transition is valid by checking if no other transition
 * has the same source state and transition input. If transition is
 * invalid, an error message is displayed.
 *
 * @param parent the parent component for the message dialog boxes
 * @param fa the current FA
 * @param source the source state for the transition
 * @param input the transition input
 *
 * @return true if transition is valid; false otherwise
 */
 
 public static boolean isValidTransition(Component parent, FA fa,
 State source, char input) 
 {
  String[] args = new String[2];
  args[0] = source.getName();
  args[1] = String.valueOf(input);

  /* if a transition from the source state on the same input
  * already exists, then transition is not valid
  */

  if (transitionExists(fa, source, input))
  {
   displayError(parent, 503, args);
   return false;
  }
  return true;
 }

 /**
 * Returns the warning message corresponding with the warning code.
 * Some warning messages require certain arguments.
 *
 * @param warningCode the warning code
 * @param args string arguments
 *
 * @return the string containing the warning message
 */

 public static String getWarningMsg(int warningCode, String[] args) 
 {
  String w = Integer.toString(warningCode);
  int n = Integer.valueOf(w.substring(2, 3)).intValue();
  return warningMsg[n - 1];
 }

 public static int checkTypeOfFA(FA fa)
 {
 	//Transition[] t = fa.getTransitionList();
 	int case_combination = 0;
 	int response = 0;
 	String msg = new String();
 	String title = "Finite Automaton Type";
 	case_combination += findDuplicateTransition(fa);
 	case_combination += findEpsilonTransition(fa);
 	case_combination += findIncompleteTransition(fa);
 	//JOptionPane.showConfirmDialog(null, case_combination);
 	switch(case_combination)
 	{
 		case 0: 		
 			msg += "The finite automaton will be set as a DFA.";
 			response = JOptionPane.showConfirmDialog(null,
 			msg,title,JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
 			if(response == 0)
 			{
 			 WorkSpacePanel.setFaType("dfa");	
 			} 	
 			else 
 			{
 			 WorkSpacePanel.setFaType("fa");
 			}		
 			break;
 		case 1:
 		case 2:
 		case 3:
 			msg += "The finite automaton will be set as an NFA.";				   
 			response = JOptionPane.showConfirmDialog(null,
 			msg,title,JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);				   
 			if(response == 0)
 			{
 			 WorkSpacePanel.setFaType("nfa");	
 			}
 			else 
 			{
 			 WorkSpacePanel.setFaType("fa");
 			}
 			break;
 		case 4:
 		case 5:
 		case 6:
 		case 7:
 			msg += "The finite automaton will be set as an eNFA.";
 			response = JOptionPane.showConfirmDialog(null,
 			msg,title,JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
 			if(response == 0)
 			{
 			 WorkSpacePanel.setFaType("enfa");		
 			}
 			else 
 			{
 			 WorkSpacePanel.setFaType("fa");
 			}
 			break; 		
 		default:
 		 	break;
 		
 		
 	}
	return response;
 	
 }
 
 public static int findDuplicateTransition(FA fa)
 {
  for (int i = 0; i < fa.getTransitionCount(); i++) 
  {
   for (int j = 0; j < fa.getTransitionCount(); j++) 
   {
    if (i != j) 
    {     
     Transition t1 = fa.getTransitionAt(i);
     Transition t2 =  fa.getTransitionAt(j);
     //JOptionPane.showConfirmDialog(null,"duplicate" + t1.getInput() + t2.getInput() );
     if ((t1.getSource().equals(t2.getSource()))&& (t1.getInput() == t2.getInput())) 
     {
      return 2;
     }
    }
   }
  }
  return 0;
 	
 }
 
 public static int findEpsilonTransition(FA fa)
 {
 	for(int i = 0;i<fa.getTransitionCount(); i++)
 	{
 		Transition t = fa.getTransitionAt(i);
 		//JOptionPane.showConfirmDialog(null,"epsilon" + t.getInput() );
 		if(t.getInput() == '')
 		{ 			
 			return 4;
 		}
 	}
 	return 0;
 }
 
 public static int findIncompleteTransition(FA fa)
 {
  State[] states = fa.getStateList();
  char[] alphabet = fa.getAlphabet();  
  for (int i = 0; i < states.length; i++) 
  {
   for (int j = 0; j < alphabet.length; j++) 
   {
   	//JOptionPane.showConfirmDialog(null,"incomplete" + alphabet[j] );     
    if (!findtransitionExists(fa, states[i], alphabet[j])) 
    {
     return 1;
    } 
   }
  }
  return 0;
 }
 
 
 /**
 * Converts the FA to FA specification format and saves it as text
 * into a file.
 *
 * @param file the file into which the fa is exported
 * @param fa the current fa to be exported
 *
 * @return true on successful export; false if an error has occurred
 */
public static boolean ExportFA(File file, FA fa) 
 {
  State[] states = fa.getStateList();
  Transition[] transitions = fa.getTransitionList();  
  char[] temp_InputAlphabet = fa.getAlphabet();
  char[] my_ABC= new char[temp_InputAlphabet.length - 1]; 
  String faName = fa.getName();
  String typeofFA = null;
  
  if(WorkSpacePanel.isItDFA())
  {
   typeofFA = "dfa";
  }
  else if(WorkSpacePanel.isItNFA())
  {
   typeofFA = "nfa";
  }
  else if(WorkSpacePanel.isIteNFA())
  {
   typeofFA = "enfa";
  }
  else if(WorkSpacePanel.isItFA())
  {
   typeofFA = "fa";
  }
    
  for (int i = 0; i < temp_InputAlphabet.length; i++)
  {
    if(i==0&&temp_InputAlphabet[i]=='')
    {
      for (int ctr = 0; ctr < temp_InputAlphabet.length-1; ctr++)
      {
       my_ABC[ctr]=temp_InputAlphabet[ctr+1];
      }
  	  fa.setAlphabet(my_ABC);
      break;
    }
  } 
  
  char[] InputAlphabet = fa.getAlphabet();  
  int rangectr = 1;
  int beginrange = 0;
  int endrange = 0;
  int IActr = 0;
  boolean dashforRangePrint = false;
  boolean possRangeFound = false;
  boolean dashfound = false;
  try 
  {
   FileOutputStream fout = new FileOutputStream(file);
   PrintStream myout = new PrintStream(fout);
   myout.println("FASPEC");
   myout.println("fa " + faName + ";");
   myout.println("type " + typeofFA + ";");
   myout.print("input_alphabet [");
   Arrays.sort(InputAlphabet);
   int DigitLetterCtr = 0;
   int NotReqdCtr = 0;
   int ReqdCtr = 0;
   /**
   * Determines the number of symbols which are 1.
   * digits/letters 2. does not require backslash 3. requires
   * backslash
   */
  for (int i = 0; i < InputAlphabet.length; i++)
  {
   if (Character.isLetterOrDigit(InputAlphabet[i]) == true) 
   {
    DigitLetterCtr++;
   } 
   else if (isBackslashCharacter(InputAlphabet[i]) == false) 
   {
    NotReqdCtr++;
   }
   else 
   {
    ReqdCtr++;
   }
  }
  /**
  * The next codes copy the Input Alphabet into 3 separate
  * arrays, one for each of the 3 criteria above. They are
  * individually sorted and recopied back into the Input
  * Alphabet.
  */
  char[] DigitLetter = new char[DigitLetterCtr];
  char[] NotReqd = new char[NotReqdCtr];
  char[] Reqd = new char[ReqdCtr];
  DigitLetterCtr = 0;
  NotReqdCtr = 0;
  ReqdCtr = 0;
  for (int i = 0; i < InputAlphabet.length; i++) 
  {
   if (Character.isLetterOrDigit(InputAlphabet[i]) == true) 
   {
    DigitLetter[DigitLetterCtr++] = InputAlphabet[i];
   } 
   else if ((isBackslashCharacter(InputAlphabet[i]) == false) && (InputAlphabet[i]!='') )
   {
    NotReqd[NotReqdCtr++] = InputAlphabet[i];
   }
   else
   {
    Reqd[ReqdCtr++] = InputAlphabet[i];
   }
   
  }
  Arrays.sort(DigitLetter);
  Arrays.sort(NotReqd);
  Arrays.sort(Reqd);
  for (int i = 0; i < DigitLetter.length; i++) 
  {
   InputAlphabet[IActr++] = DigitLetter[i];
  }
  for (int i = 0; i < NotReqd.length; i++) 
  {
   InputAlphabet[IActr++] = NotReqd[i];
  }
  for (int i = 0; i < Reqd.length; i++) 
  {
   InputAlphabet[IActr++] = Reqd[i];
  }
  
  for (int i = 0; i < InputAlphabet.length; i++) 
  {
   if (Character.isLetterOrDigit(InputAlphabet[i]) == false) 
   {
    /**
    * if symbol is not a digit/letter, print if not dash.
    * Ranges are not possible.
   */
    if (InputAlphabet[i] == '-') 
    {
     dashfound = true;
    } 
    else if (InputAlphabet[i] == ' ') 
    {
     myout.print("\\s");
    } 
    else if (InputAlphabet[i] == '') 
    {
     myout.print("");
    } 
    else if (isBackslashCharacter(InputAlphabet[i]) == true) 
    {
     myout.print("\\" + InputAlphabet[i]);
    } 
    else 
    {
     myout.print(InputAlphabet[i]);
    }
    rangectr = 1;
    possRangeFound = false;
   }  
   else if (i == (InputAlphabet.length - 1)) 
   {
    /**
    * print last symbol in the alphabet
    */
    myout.print(InputAlphabet[i]);
   } 
   else if ((int) InputAlphabet[i] == (int) (InputAlphabet[i + 1] - 1)) 
   {
    /**
    * if 2 adjacent cells contain consecutive symbols,
    * ranges are allowed and checked when printing.
    */
    if (possRangeFound == false) 
    {
     possRangeFound = true;
     beginrange = i;
     myout.print(InputAlphabet[i]);
    }
    rangectr++;
    endrange = i + 1;
    if (rangectr >= 3) 
    {
     /**
     * if at least 3 consecutive symbols, a dash is
     * printed in place of the 2nd until the range
     * ends.
     */
     if (dashforRangePrint == false) 
     {
      dashforRangePrint = true;
      myout.print("-");
     } 
    }
   } 
   else 
   {
    dashforRangePrint = false;
    possRangeFound = false;
    rangectr = 1;
    myout.print(InputAlphabet[i]);
   }
  }
  /**
  * prints a dash at the end if it was present
  */
  if (dashfound == true) 
  { 
   myout.print("-");
  }
  
  myout.println("];");
 
  /**
  * the next codes print the states
  */

  myout.print("states ");
  String tempName;
  boolean tempFlag = false;
  //locate Starting State
  for (int i = 0; i < fa.getStateCount(); i++) 
  {
   tempName = states[i].getName();
   if (tempName.equals(fa.getStartingState().getName())) 
   {
    tempFlag = fa.getStartingState().getAcceptFlag();
    if (tempFlag == true) 
    {
     myout.print("*");
    }
    myout.print(tempName);
    if((fa.getStateCount()>1) && (i!=(fa.getStateCount()-1)))
    {
     myout.print(",");	
    }
    break;
   }
  }
 
   for (int i = 0; i < fa.getStateCount(); i++) 
   {
    tempName = states[i].getName();
    tempFlag = states[i].getAcceptFlag();
    /**
    * if the state is accepting, print an asterisk before the
    * name
    */
    if (tempName != fa.getStartingState().getName()) 
    {
     if (tempFlag == true) 
     {
      myout.print("*");
     }
     myout.print(tempName);
     /**
     * if the state is not the last in the list, print a
     * comma.
     */
     if (i < (fa.getStateCount() - 1)) 
     {
      myout.print(",");
     }
    }
   }
   myout.println(";");
 


// prints the transitions
   myout.println("transitions");
   myout.println("{");
   for (int i = 0; i < fa.getTransitionCount(); i++) 
   {
    String a = transitions[i].getSource().getName();
    String b = transitions[i].getDestination().getName();
    char c = transitions[i].getInput();
    myout.print("\t" + a + ",");
  
    /**
    * if backslash is required for the input symbol, print a
    * backslash
    */
    if (isBackslashCharacter(c) == true) 
    {
     myout.print("\\");
     /**
     * if symbol was space, the backslash has been
     * printed, print 's'
     */
    }
    if (c == ' ') 
    {
     myout.print("s");
     /**
     * otherwise, whether backslash required or not, print
     * the input symbol
     */
    }
    if (c == '') 
    {
     myout.print("\\e");
     /**
     * otherwise, whether backslash required or not, print
     * the input symbol
     */
    } 
    else 
    {
     myout.print(c);
    }
    myout.println(" : " + b + ";");
   }
   myout.println("}");
   myout.println("fa_end;");
  } 
  catch (IOException e) 
  {
   return false;
  }
  return true;
 }
 
/**
* Checks if the FA has at least one accepting state.
*
* @param fa the current FA
*
* @return true if at least one accepting state exists; false if none
*/

 public static boolean doesAcceptingStateExist(FA fa) 
 {
  State[] states = fa.getStateList();
  for (int i = 0; i < states.length; i++) 
  {
   if (states[i].getAcceptFlag() == true) 
   {
    return true;
   }
  }
  return false;
 }

 /**
 * Converts a character array to a Vector of Character objects.
 * 
 * @param c the character array to be converted
 *
 * @return a Vector of Character objects, each with character values
 * equal to those in the character array
 */
 public static Vector charArraytoVector(char[] c) 
 {
  Vector v = new Vector();
  for (int i = 0; i < c.length; i++) 
  {
   v.addElement(new Character(c[i]));
  }
  return v;
 }

 /**
 * Compares two input alphabets.
 *
 * @param oldIA the first input alphabet
 * @param newIA the new input alphabet
 *
 * @return a string representation of the inputs which exist in the
 * first alphabet but not in the second
 */
 public static String compareAlphabet(char[] oldIA, char[] newIA) 
 {
  /* convert the alphabets to vector form first */
  Vector oldV = charArraytoVector(oldIA);
  Vector newV = charArraytoVector(newIA);
  
  /* remove from the old alphabet all exisiting the characters
  * in the new alphabet
  */

  oldV.removeAll(newV);
  /* the old vector now contains characters not found in new
  * vector. these have been erased when the alphabet was edited,
  * so group these characters together for display.
  */
 
  char[] c = vectortoCharArray(oldV);
  String s = new String();
  
  /* format some characters for display */
  for (int i = 0; i < c.length; i++) 
  {
   if (isBackslashCharacter(c[i]))
   {
    s = s + "\\\\";
   }
   if (c[i] == ' ') 
   {
    s = s + "\\\\s";
   }
   else 
   {
    s = s + String.valueOf(c[i]);
   }
  }
  return s;
 }

 /**
 * Displays an error message for the specified error code.
 *
 * @param parent the parent component for the message dialog boxes
 * @param code the error code
 */
 
 public static void displayError(Component parent, int code) 
 {
  displayError(parent, code, NO_ARGS);
 }

 /**
 * Displays an error message for the specified error code.
 *
 * @param parent the parent component for the message dialog boxes
 * @param code the error code
 * @param args string arguments
 */

 public static void displayError(Component parent, int code, String[] args) 
 {
  JOptionPane.showMessageDialog(parent, getErrorMsg(code, args),
  "Error " + String.valueOf(code),
  JOptionPane.ERROR_MESSAGE);
 }

 /**
 * Displays a message with the specified message code.
 *
 * @param parent the parent component for the message dialog boxes
 * @param code the message code
 */
 public static void displayInfo(Component parent, int code) 
 {
  displayInfo(parent, code, NO_ARGS);
 }

 /**
 * Displays a message with the specified message code.
 *
 * @param parent the parent component for the message dialog boxes
 * @param code the message code
 * @param args string arguments
 */

 public static void displayInfo(Component parent, int code, String[] args) 
 {
  JOptionPane.showMessageDialog(parent, getInfoMsg(code, args));
 }

 /**
 * Displays a warning with the specified warning code.
 *
 * @param parent the parent component for the message dialog boxes
 * @param code the warning code
 *
 * @return FAChecker.OK if the OK button on the confirmation dialog
 * box is clicked; FAChecker.CANCEL if the cancel button is
 * clicked.
 */
 
 public static int displayWarning(Component parent, int code) 
 {
  return displayWarning(parent, code, NO_ARGS);
 }

 /**
 * Displays a warning with the specified warning code.
 *
 * @param parent the parent component for the message dialog boxes
 * @param code the warning code
 * @param args string arguments
 *
 * @return FAChecker.OK if the OK button on the confirmation dialog
 * box is clicked; FAChecker.CANCEL if the cancel button is
 * clicked.
 */
 
 public static int displayWarning(Component parent, int code, String[] args) 
 {
  return JOptionPane.showConfirmDialog(parent, getWarningMsg(code, args),
  "Warning " + String.valueOf(code),JOptionPane.OK_CANCEL_OPTION,
  JOptionPane.WARNING_MESSAGE);
 }

 public static char epsilonChar()
 {
  String epsilon = "";
  return (epsilon.charAt(0));
 }
 /**
 * Expands an input alphabet string to an array of input characters by
 * converting ranges into actual characters. If a range is invalid,
 * then the alphabet is invalid.
 *
 * @param alphastring the input alphabet string
 *
 * @return a character array representing the actual input alphabet
 */

 public static char[] expandAlphabetString(String alphastring) 
 {
  /* initiate vector for expandable character list */
  Vector v = new Vector();
  String epsilon = "";  
  Character c;
  char start;
  char end;
  for (int i = 0; i < alphastring.length(); i++) 
  {
   /* get each character from the string */
   c = new Character(alphastring.charAt(i));
      
   if(((WorkSpacePanel.isIteNFA())||(WorkSpacePanel.isItFA())) && (i==0) )
   {   	 
   	v.addElement(epsilonChar());   	
   }
   if(( (c.charValue() < 32) || (c.charValue() > 126) ) && c.charValue()!=137)
   {
   	return null;
   }
   /* check if character is already added to the input vector */
   if (!v.contains(c)) 
   {   	
    if (((c.charValue() == '-') && (i == 0))|| (i == (alphastring.length() - 1))) 
    {
     v.addElement(c);
     /* if a dash '-' has been found somewhere between the
     * first and the last character in the string, then
     * a possible range has been detected
     */
    } 
    else if (c.charValue() == '-') 
    {
     start = alphastring.charAt(i - 1);
     end = alphastring.charAt(i + 1);
     /* check if range is valid */
     if (start < end)
     {
      if( (start>=48 && end<=57) || (start>=65 && end<=90) || (start>=97 && end<=122))
      {
       for (char j = (char) (start + 1); j <= (char) (end-1); j++) 
       {
        v.addElement(new Character(j));              
       }	
      }
      else
      {
       return null;	
      }      
     } 
     else 
     {
      return null;
     }     
    /* just add all other characters*/
    } 
    else 
    {
     v.addElement(c);
     
    }
   }
  }
  /* return as character array */
  return vectortoCharArray(v);
 }

 /**
 * Checks if transition with the given source state and transition
 * input already exists in the FA.
 *
 * @param fa the working fa
 * @param source the source state
 * @param input the transition input
 *
 * @return true if transition exists; false if not
 */

 public static boolean transitionExists(FA fa, State source,char input) 
 {
  String[] args = new String[2];
  if( WorkSpacePanel.isItDFA() )
  {
   for (int i = 0; i < fa.getTransitionCount(); i++)
   {
    Transition transition = fa.getTransitionAt(i);
    if ((transition != null) && (source == transition.getSource())
    && (input == transition.getInput())) 
    {
     return true;
    }
   }	
  }  
  return false;
 }

 public static boolean findtransitionExists(FA fa, State source,char input) 
 {
  String[] args = new String[2];
  for (int i = 0; i < fa.getTransitionCount(); i++)
  {
   Transition transition = fa.getTransitionAt(i);
   if ((transition != null) && (source == transition.getSource())
   && (input == transition.getInput())) 
   {
    return true;
   }
  }	
  return false;
 }
 /**
 * Converts a vector to a character array.
 *
 * @param v the vector to be converted
 *
 * @return a character array copy of the vector
 */
 
 public static char[] vectortoCharArray(Vector v) 
 {
  char[] c = new char[v.size()];
  for (int i = 0; i < v.size(); i++) 
  {
   c[i] = ((Character) v.elementAt(i)).charValue();
  }    
  return c;
 }
 
}
