 

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
* Implements a set of methods for parsing a text file in FA specification
* format and importing it into an object of FA type.
*/
public class FAParser 
{ 
 //~ Instance variables ************************************************
 /** The parent component of this class. */
 private Component container;

 /** The FA name. */
 private String FAName = null;
 //private String FAType = null;

  /** The list of lines in the FA text file. */
 private Vector FA = new Vector();

 /** The list of input alphabet. */
 private Vector alphabet = new Vector();

 /** The list of states in the FA. */
 private Vector stateList = new Vector();

 /** The list of transitions in the FA. */
 private Vector transitionList = new Vector();

 /** The number of lines the FA file. */
 private int lineCount = 0;
 
 private static String FA_Type = new String();

 //~ Constructors ******************************************************
 /**
 * The sole constructor for the class.
 *
 * @param in the file reader where the FA file is buffered
 * @param c the parent component of this class
 */
 public FAParser(LineNumberReader in, Component c) 
 {
  String s;
  container = c;
  while ((s = getCurrentLine(in)) != null) 
  {
   s = s.trim();
   if (s.length() >= 1) 
   {
    FA.addElement(s);
    this.lineCount++;
   }
  }
 }
 
 //~ Methods ***********************************************************
 /**
 * Returns the parsed input alphabet as an array of input characters.
 *
 * @return the input alphabet
 */
 
 public char[] getAlphabet() 
 {
  parseAlphabet();
  return FAChecker.vectortoCharArray(alphabet);
 }
 
 /**
 * Checks if an input character should require a backslash preceding
 * it.
 *
 * @param c the input character to check
 *
 * @return true if the input character requires a preceding backslash;
 * false otherwise
 */

 public boolean isBackslashCharacter(char c) 
 {
  if ((c == 's') || (c == 'e') || (c == ']') || (c == '[') || (c == '|') || (c == ';') || (c == ':') || (c == '{') || (c == '}') || (c == '\\') || (c == '"')) 
  {
   return true;
  }
  return false;
 }

 /**
 * Returns the parsed FA name as a string.
 *
 * @return the FA name
 */
 public String getFAName() 
 {
  parseFAName();
  return FAName;
 }

 public static String getFAType() 
 {
  return FA_Type;
 }

 /**
 * Checks if an error has occurred while parsing the file into a FA.
 * An error will occur if at least one of the following conditions
 * are not met: the file is a text file in FA specification format,
 * the FA is in fact a FA, and the FA is a valid FA.
 *
 * @return true if an error has occurred; false if none
 */
 public boolean isError() 
 {
  /* check if the file is a valid FA file, the FA is indeed a FA,
  * and the FA is valid. */
  if(isValidFile() && isFA())
  {   
   if(isValidFA())
   {
   	return false;
   }
  }
  return true;  
 }

 /**
 * Check if an input character can be found in the parsed input
 * alphabet.
 *
 * @param input the input to be checked
 *
 * @return true if the input is in the alphabet; false otherwise
 */

 public boolean isInputValid(char input) 
 {
  return alphabet.contains(new Character(input)); 
 } 

 /**
 * Returns a set of error messages or FA info messages, depending on
 * the error generated by the parser.
 *
 * @return the error messages if an error has occurred while parsing
 * the file; a FA information message on a successful
 * parsing
 */
 public String getMessage() 
 {
  String msg = new String();
  
  /* get error message if file is not an FA text file */
  if (!isValidFile()) 
  {
   msg = "Invalid file.\n";
   
   /* get error message if not a FA */
  } 
  else if (!isFA()) 
  {
   msg = "Invalid file. FA is not of type FA.\n";
   /* get error messages for every invalid part of the FA */
  } 
  else if (!isValidFA()) 
  { 
   msg = "Error reading FA specifications.\n";
   msg += ((parseFAName())? "": "* Invalid FA name " + "(must be an identifier).\n");
   msg += ((parseAlphabet())? "" : "* Invalid input alphabet.\n");
   msg += ((parseStates())? "" : "* Invalid state list.\n");
   msg += ((parseTransitions())? "" : "* Invalid transition function.\n");
   msg += ((duplicateTransitions())? "Duplicate transitions exist.\n" : "");
   
   /* get the FA info message if it is valid */
  } 
  else 
  {
   char[] ia = FAChecker.vectortoCharArray(alphabet);
   char[] ia_temp = new char[ia.length - 1];
   for(int i = 0;i<ia.length - 1; i++)
   {
   	ia_temp[i] = ia[i+1];
   }   
   msg = "FA successfully imported.\n\n";
   msg += "FA Name: " + FAName + "\n";
   msg += "FA type: " + FA_Type+ "\n";
   if(FA_Type != "enfa")
   {
   	Arrays.sort(ia);
   	msg += ("Input alphabet: " + String.valueOf(ia) + "\n");
   }
   else if(FA_Type == "enfa")
   {
   	Arrays.sort(ia_temp);
   	msg += ("Input alphabet: " + String.valueOf(ia_temp) + "\n");
   }   
   msg += "States: \n";
   for (int i = 0; i < stateList.size(); i++) 
   {
    State s = (State) stateList.elementAt(i);
    msg += (" " + s.getName());
    msg += ((s.getAcceptFlag())? " (accepting)\n" : "\n");
   }
   msg += "Transition function \u03B4: \n";
   for (int i = 0; i < transitionList.size(); i++) 
   {
   	Transition t = (Transition) transitionList.elementAt(i);
    msg += " " + '\u03B4' + "(" + t.getSource().getName();
    msg += ", " + t.getInput() + ") = ";
    msg += t.getDestination().getName() + "\n";
   }
   msg += "\n" + "The states and transitions of the FA have\n";
   msg += " been automatically drawn on the workspace. You may\n";
   msg += "move them for the purpose of visual clarity.\n";
  }
  return msg;
 }
 
 /**
 * Checks if another state generated by the parser already use the
 * coordinates specified.
 *
 * @param x the x-coordinate for the circle representation of the
 * state
 * @param y the y-coordinate for the circle representation of the
 * state
 *
 * @return true if the coordinates are occupied; false otherwise
 */
 public boolean isOccupied(int x, int y) 
 {
  double d;
  for (int i = 0; i < stateList.size(); i++) 
  {
   State s = (State) stateList.elementAt(i);
   d = Point.distance((double) s.getX(), (double) s.getY(), (double) x, (double) y);
   if (d <= 25) 
   {
    return true;
   }
  }
  return false;
 }

 /**
 * Returns the list of parsed states as an array of State objects.
 *
 * @return the array of states
 */
 public State[] getStateList() 
 {
  parseStates();
  return (State[]) stateList.toArray(new State[stateList.size()]);
 }

 /**
 * Returns the list of parsed states as an array of Transition
 * objects.
 *
 * @return the array of transitions
 */
 public Transition[] getTransitionList() 
 {
  parseTransitions();
  return (Transition[]) transitionList.toArray(new Transition[transitionList.size()]);
 }

 /**
 * Checks for duplicate transitions.
 *
 * @return true if at least one duplicate transition is found; false
 * if none
 */
 public boolean duplicateTransitions() 
 {
  for (int i = 0; i < transitionList.size(); i++) 
  {
   for (int j = 0; j < transitionList.size(); j++) 
   {
    if (i != j) 
    {
     Transition t1 = (Transition) transitionList.elementAt(i);
     Transition t2 =  (Transition) transitionList.elementAt(j);
     if ((t1.getSource().equals(t2.getSource()))&& (t1.getInput() == t2.getInput())) 
     {
      return true;
     }
    }
   }
  }
  return false;
 }

 /**
 * Returns the current line being scanned in the file reader buffer.
 *
 * @param in the file reader buffer
 *
 * @return the current line being scanned
 */
 private String getCurrentLine(LineNumberReader in) 
 {
  try 
  {
   String line = in.readLine();
   if ((line != null) && (line.length() > 0)) 
   {
    int n1 = line.indexOf("//");
    if (n1 >= 0) 
    {
     line = line.substring(0, n1);
    }
    line = line.trim();
   }
   return line;
  }
  catch (IOException e) 
  {
   return null;
  }
 }

 /**
 * Checks if the parsed FA is a FA.
 *
 * @return true if it is a FA; false otherwise
 */
 private boolean isFA() 
 {
  if (getLine(2).substring(5, 8).equalsIgnoreCase("dfa")) 
  {   
   FA_Type = "dfa";   
   return true;
  }
  else if(getLine(2).substring(5, 8).equalsIgnoreCase("nfa"))
  {
   FA_Type = "nfa";   
   return true;	
  }
  else if(getLine(2).substring(5, 9).equalsIgnoreCase("enfa"))
  {  
   FA_Type = "enfa";   
   return true;	
  }  
  return false;
 }

 /**
 * Returns the line specified by the line number from the parsed FA
 * text lines.
 *
 * @param n the line number
 *
 * @return the string containing the text in the line
 */
 private String getLine(int n) 
 {
  return (String) FA.elementAt(n);
 }

 /**
 * Returns the State object using the specified name.
 *
 * @param name the name of the state
 *
 * @return the State using the name
 */
 private State getState(String name) 
 {
  for (int i = 0; i < stateList.size(); i++) 
  {
   State s = (State) stateList.elementAt(i);
   if (s.getName().equals(name)) 
   {
    return s;
   }
  }
  return null;
 }
 
 /**
 * Checks if the parsed FA is valid. The parsed FA is valid if all
 * of these conditions are met: all elements of the FA (name, input
 * alphabet, state list, and transition list) have been successfully
 * parsed, and there are no duplicate transitions.
 *
 * @return true if FA is valid; false otherwise
 */
 private boolean isValidFA() 
 {
  if(FA_Type == "dfa" )
  {
   
   if (parseFAName() && parseAlphabet()  && parseStates() && parseTransitions() && !duplicateTransitions()) 
   {   	
    return true;
   }
  }
  else if(FA_Type == "nfa" || FA_Type == "enfa")
  {
   if (parseFAName()  && parseAlphabet() && parseStates() && parseTransitions()) 
   {   	
    return true;
   }
  }  
  return false;
 }

 /**
 * Checks if the file is a valid text file in FA specification format.
 *
 * @return true if file is an FA file; false otherwise
 */
 private boolean isValidFile() 
 {
  if ((getLine(0).substring(0, 6).equals("FASPEC")) && (getLine(1).substring(0, 2).equals("fa"))&& (getLine(2).substring(0, 4).equals("type"))&& (getLine(3).substring(0, 14).equals("input_alphabet"))&& (getLine(4).substring(0, 6).equals("states"))&& (getLine(5).trim().equals("transitions")) && (getLine(6).trim().equals("{"))&& (getLine(lineCount - 2).trim().equals("}"))&& (getLine(lineCount - 1).trim().equals("fa_end;"))&& (getLine(3).substring(15, 16).equals("["))&& (getLine(3).endsWith("];"))) 
  {
   for (int i = 1; i < lineCount; i++) 
   {
    if ((i != 5) && (i != 6) && (i != lineCount - 2)) 
    {
     if (!getLine(i).endsWith(";")) 
     {
      return false;
     }
    }
   }
   return true;
  }
  return false;
 }

 /**
 * Checks if a line is a valid line (only for lines that should end
 * with a semi-colon).
 *
 * @param s the line
 * 
 * @return true if line is valid; false otherwise
 */
 private boolean isValidLine(String s) 
 {
  if (s.endsWith(";")) 
  {
   return true; 
  }
  return false;
 }

 /**
 * Parses the FA text to generate the input alphabet.
 *
 * @return true on a succesful parse; false if an error occurs
 */
 private boolean parseAlphabet() 
 {
  boolean backSlashEnabled = false;
  String a = getLine(3).substring(16, getLine(3).length() - 2);
  Character x;
  alphabet.removeAllElements();
  
  for (int i = 0; i < a.length(); i++) 
  {
   if(i==0)
   {
    if(FA_Type=="enfa")
    { 
  	 x = new Character('');
  	 alphabet.addElement(x);
    }
   }
   char c = a.charAt(i);
   x = new Character(c);
   if (c == ' ') 
   {
   } 
   else if (c == '-') 
   {
    if ((i == 0) || (i == a.length() - 1)) 
    {
     if (!alphabet.contains(x)) 
     {
      alphabet.addElement(x);
     }
    } 
    else 
    {
     char s = ((Character) alphabet.lastElement()).charValue();
     char e = a.charAt(i + 1);
     if (s < e) 
     {      
      for (char j = (char) (s + 1); j <= e; j++) 
      {
       x = new Character(j);
       if (!alphabet.contains(x)) 
       {
        alphabet.addElement(x);
       }
      }
      i++;
     } 
     else 
     {
      return false;
     }
    }
    backSlashEnabled = false;
   } 
   else if (c == '\\') 
   {
    i = i + 1;
    c = a.charAt(i);
    if (isBackslashCharacter(c)) 
    {
     c = (c == 's')? ' ' : c;
     c = (c == ']')? ']' : c;
     c = (c == '[')? '[' : c;
     //c = (c == 'e')? '' : c;
     x = new Character(c);
     if (!alphabet.contains(x)) 
     {
      alphabet.addElement(x);
     } 
    } 
    else 
    {
     return false;
    }
    backSlashEnabled = true;
   } 
   /*else if (isBackslashCharacter(c) && (c != 's')) 
   {
    if (backSlashEnabled) 
    {
     if (!alphabet.contains(x)) 
     {
      alphabet.addElement(x);
     }
    } 
    else 
    {
     return false;
    }
   }*/ 
   else 
   {
    if (!alphabet.contains(x)) 
    {
     alphabet.addElement(x);
    }
    backSlashEnabled = false;
   }
  }
  return true;
 }

 /**
 * Parses the FA text to generate the FA name.
 *
 * @return true on a succesful parse; false if an error occurs
 */
 private boolean parseFAName() 
 {
  String s = getLine(1);
  s = s.substring(3, s.length() - 1);
  if (!FAChecker.isIdentifier(s)) 
  {
   return false;
  }
  FAName = s;
  return true;
 }

 /*private boolean parseFAType() 
 {
  String s = getLine(2);
  s = s.substring(5, s.length() - 1);
  if (!FAChecker.isIdentifier(s)) 
  {
   return false;
  }
  FA_Type = s;
  setWSPsetFaType(FAName);
  return true;
 }*/

 public static void setWSPsetFaType(String FAtype)
 {
  //WorkSpacePanel.setFaType(FAtype);
 }

 /**
 * Parses the FA text to generate the state list.
 *
 * @return true on a succesful parse; false if an error occurs
 */
 private boolean parseStates() 
 {
  String s = getLine(4);
  String stateString = s.substring(7, s.length() - 1);
  String[] list = stateString.split(","); //array of state names
  String name;
  boolean acceptFlag = false;
  boolean stateSet = false;
  int x = 0;
  int y = 100;
  stateList.removeAllElements();
  for (int i = 0; i < list.length; i++) 
  {
   s = list[i].trim(); //gets start of string 
   if (s.startsWith("*")) 
   { 
    acceptFlag = true;
    name = s.substring(1);
   } 
   else 
   {
    acceptFlag = false;
    name = s;
   }
   if (!FAChecker.isIdentifier(name)) 
   {
    return false;
   }
   State state = new State(name, acceptFlag);
   if (!stateList.contains(state)) 
   {
    stateList.addElement(state);
   }
   Dimension panelSize = container.getPreferredSize();
   int panelWidth = panelSize.width;
   int panelHeight = panelSize.height;  
   stateSet = false;
   while (!stateSet) 
   {  
    x += 100;
    if (x > (panelWidth - 100)) 
    {
     x = 100;
     y += 100;
    }
    if (y < (panelHeight - 100)) 
    {
     if (!isOccupied(x, y)) 
     {
      state.setCoordinates(x, y);
      stateSet = true;
     }
    } 
    else 
    {
     state.setCoordinates(panelWidth - 100,panelHeight - 100);
     stateSet = true;
    }
   }
  }
  return true;
 }
 
 /**
 * Parses the FA text to generate the transition list.
 *
 * @return true on a succesful parse; false if an error occurs
 */
 private boolean parseTransitions() 
 {
  State src;
  State dst;
  String srcName;
  String dstName;
  String input; 
  char inp;  
  String[] s = new String[lineCount - 2 - 7];
  /*two lines from the 
   *bottom and 7 lines from the top are not included
   *nine lines in the FASPEC are not for transitions*/
  String msg = new String();
  transitionList.removeAllElements();
  for (int i = 0; i < s.length; i++) 
  {
   s[i] = getLine(7 + i).trim();
   srcName = s[i].substring(0, s[i].indexOf(',')).trim();
   dstName = s[i].substring(s[i].lastIndexOf(':') + 1,s[i].length() - 1).trim();
   src = getState(srcName);
   dst = getState(dstName);
   if ((src == null) || (dst == null)) 
   {
    return false;
   }
   input = s[i].substring(s[i].indexOf(',') + 1,s[i].lastIndexOf(':')).trim();
   if ((input.length() < 1)) 
   {
    return false;
   } 
   else if (input.length() == 2) 
   {
    if ((input.charAt(0) == '\\') && isBackslashCharacter(input.charAt(1))) 
    {
     input = String.valueOf(input.charAt(1));
     if (input.equals("s")) 
     {
      input = " ";
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("e"))
     {
      input = "";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("["))
     {
      input = "[";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("]"))
     {
      input = "]";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("{"))
     {
      input = "{";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("}"))
     {
      input = "}";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("\""))
     {
      input = "\"";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("|"))
     {
      input = "|";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals(":"))
     {
      input = ":";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals(";"))
     {
      input = ";";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else if(input.equals("\\"))
     {
      input = "\\";	
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
     else
     {
      input = String.valueOf(input.charAt(1));
      transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
     }
    } 
    else 
    {
     return false;
    }
   } 
   else if (isBackslashCharacter(input.charAt(0)) && (input.length()>1) ) 
   {
    return false;
   }
   else if (!isInputValid(input.charAt(0))) 
   {
    return false;
   }
   else 
   {
   	//input = input;
   	transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
   }
   //transitionList.addElement(new Transition(src, dst, input.charAt(0), 0));
  }
  return true;
 }
}
