 

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
* Implements an input dialog box which accepts a string as input string
* for simulating or testing a FA. The input string is checked for
* validity by checking if it is in the input alphabet. (NOTE: the empty
* string is also in the input alphabet, so it is valid). If the input
* string is valid, then the simulation may proceed. Simulation is done
* by checking each input in the string in sequence against the
* transition table. The transition input and the current state (source
* state) is passed to a function to return the transition and then the
* destination state is read from this transition. The simulator proceeds
* to this destination state, and reads the next input in the input
* string. Once the last input has been read, it checks the current state
* if it is accepting or not. If it is accepting, then the input string
* is accepted by the FA; otherwise, the string is not in the language
* of the FA. There are two methods to simulate the FA: one is the
* quick-and-dry method, and the other is a state-by-state method. The
* later allows the user to simulate the input string on the FA on a
* step-by-step method. The former automatically simulates the whole
* string without user interruption.
*/
public class FASimulateDialogBox extends JDialog implements ActionListener 
{
 //~ Instance variables ************************************************
 /** The FA to be simulated. */ 
 private FA fa;

 /** The parent component of this dialog box. */
 private FAEditor container;

 /** The cancel button. */
 private JButton jbtCancel;

 /** The 'next input' button. Used for state-by-state method. */
 private JButton jbtNext;

 /** The 'play' button. Used for quick-and-dry simulation. */ 
 private JButton jbtPlay;

 /** The 'previous input' button. Used for state-by-state method. */
 private JButton jbtPrev;

 /** The start button to begin simulation. */
 private JButton jbtStart;

 /** Textfield for input string. */
 private JTextField simInput;

 /** Non-editable textfield for simulation status display. */ 
 private JTextField status;

 /** Used to score the dialog box location. */
 private Point l;

 /** The current state. */
 private State state;

 /** The input string. */
 private String input;

 /** The FA state list. */
 private Vector StateList = new Vector();

 /** The workspace where the simulation is made. */
 private WorkSpacePanel workSpacePanel; 

 /** Checks if an exit simulation action is allowed. */
 private boolean isExitAllowed = true;

 /** The current input being read. */
 private char in;

 /** The input position in the input string. */
 private int n = 0; 

 //~ Constructors ******************************************************
 /**
 * Generates the simulation dialog box.
 *
 * @param parent the parent component of this dialog box
 * @param inputSpacePanel the workspace where the simulation will be
 * shown
 */
 public FASimulateDialogBox(JFrame parent,WorkSpacePanel inputSpacePanel) 
 {
  super(parent, true);
  workSpacePanel = inputSpacePanel;
  container = (FAEditor) parent;
  setTitle("Simulate FA");
  setSize(300, 200);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  p1.add(new JLabel("Enter String to Simulate "));
  p1.add(simInput = new JTextField(10));
  JPanel pSim = new JPanel();
  pSim.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
  pSim.add(jbtPrev = new JButton(new ImageIcon("icons/previous.gif")));
  pSim.add(jbtNext = new JButton(new ImageIcon("icons/next.gif")));
  pSim.add(jbtPlay = new JButton(new ImageIcon("icons/play.gif")));
  JPanel pMain = new JPanel();
  pMain.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 5));
  pMain.add(p1);
  pMain.add(pSim);
  pMain.setBorder(new TitledBorder("Simulation Input"));
  JPanel pButtons = new JPanel();
  pButtons.setLayout(new FlowLayout(FlowLayout.RIGHT, 10, 10));
  pButtons.add(jbtStart = new JButton("Begin Simulation"));
  pButtons.add(jbtCancel = new JButton("Exit Simulation"));
  jbtPlay.setEnabled(false);
  jbtPrev.setEnabled(false);
  jbtNext.setEnabled(false);
  getContentPane().setLayout(new BorderLayout());
  getContentPane().add(status = new JTextField("Waiting to begin " + "simulation..."),BorderLayout.NORTH);
  getContentPane().add(pMain, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);
  status.setEnabled(true);
  status.setEditable(false);
  jbtStart.addActionListener(this);
  jbtCancel.addActionListener(this);
  jbtPlay.addActionListener(this);
  jbtPrev.addActionListener(this);
  jbtNext.addActionListener(this);
 }


 //~ Methods ***********************************************************
 /**
 * Determines the simulation dialog box action.
 *
 * @param e the action event generated by a click on any of the
 * buttons in the simulation dialog box
 */
 public void actionPerformed(ActionEvent e) 
 {
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtStart) 
   {
    input = simInput.getText();
    if (FAChecker.isValidInputString(container,workSpacePanel.getFa(),input)) 
    {
     Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
     Dimension boxSize = this.getSize();
     l = this.getLocation();
     this.setLocation(screenSize.width - boxSize.width - 50,50);
     simInput.setEditable(false);
     simInput.setEnabled(false);
     jbtStart.setEnabled(false);
     jbtPlay.setEnabled(true);
     jbtPrev.setEnabled(false);
     jbtNext.setEnabled(true);
     jbtCancel.setText("Cancel Simulation");
     isExitAllowed = false;
     StateList = new Vector();
     fa = workSpacePanel.getFa();
     state = fa.getStartingState();
     StateList.addElement(state);
     workSpacePanel.setSelectedState(state);
     workSpacePanel.repaint();
     if (input.length() > 0) 
     {
      n = 0;
      in = input.charAt(n);
      status.setText("Waiting for transition from state "+ state.getName() + " on input " + in+ " ... ");
     } 
     else 
     {
      status.setText("Waiting for transition from state "+ state.getName() + " on empty string... ");
     }
    }
   } 
   else if (e.getSource() == jbtNext) 
   {
   	if (input.length() > 0) 
   	{
     state = fa.getTransition((State) StateList.lastElement(), in).getDestination();
     StateList.addElement(state);
     workSpacePanel.setSelectedState(state);
     workSpacePanel.repaint();
     jbtPrev.setEnabled(true);
     n++;
     if (n < input.length()) 
     {
      in = input.charAt(n);
      status.setText("Waiting for transition from state "+ state.getName() + " on input " + in+ " ... ");
     }
    }
    if ((input.length() == 0) || (n == input.length())) 
    {
     String msg = state.getAcceptFlag()? "The input string is in the language of the FA.": "The input string is not accepted by the FA.";
     Icon msgIcon = new ImageIcon(state.getAcceptFlag()? "icons/checkmark.gif": "icons/xmark.gif");
     JOptionPane.showMessageDialog(this, msg,"FA Simulation Result",JOptionPane.INFORMATION_MESSAGE,msgIcon);
     status.setText("Waiting for another simulation");
     simInput.setEditable(true);
     simInput.setEnabled(true);
     jbtStart.setEnabled(true);
     jbtPlay.setEnabled(false);
     jbtPrev.setEnabled(false);
     jbtNext.setEnabled(false);
     jbtCancel.setText("Exit Simulation");
     isExitAllowed = true;
     this.setLocation(l);
    }
   } 
   else if (e.getSource() == jbtPrev) 
   {
    StateList.removeElementAt(StateList.size() - 1);
    state = (State) StateList.lastElement();
    n--;
    if (StateList.size() == 1) 
    {
     jbtPrev.setEnabled(false);
    } 
    else 
    {
     in = input.charAt(n);
    }
    status.setText("Waiting for transition from state " + state.getName() + " on input " + in + " ... ");
    workSpacePanel.setSelectedState(state);
    workSpacePanel.repaint();
   } 
   else if (e.getSource() == jbtPlay) 
   {
    jbtPrev.setEnabled(false);
    jbtNext.setEnabled(false);
    for (int i = n; i < input.length(); i++) 
    {
     in = input.charAt(i);
     state = fa.getTransition((State) StateList.lastElement(), in).getDestination();
     StateList.addElement(state);
     workSpacePanel.setSelectedState(state);
     workSpacePanel.repaint();
    }
    String msg = state.getAcceptFlag()? "The input string is in the language of the FA.": "The input string is not accepted by the FA.";
    Icon msgIcon = new ImageIcon(state.getAcceptFlag()? "icons/checkmark.gif": "icons/xmark.gif");
    JOptionPane.showMessageDialog(this, msg,"FA Simulation Result",JOptionPane.INFORMATION_MESSAGE,msgIcon);
    status.setText("Waiting for another simulation...");
    simInput.setEditable(true);
    simInput.setEnabled(true);
    jbtStart.setEnabled(true);
    jbtCancel.setText("Exit Simulation");
    isExitAllowed = true;
    jbtPlay.setEnabled(false);
    this.setLocation(l);
   }  
   else if (e.getSource() == jbtCancel) 
   {
    if (isExitAllowed) 
    {
     this.setVisible(false);
    }   
    else 
    {
     simInput.setEditable(true);
     simInput.setEnabled(true);
     jbtStart.setEnabled(true);
     jbtPlay.setEnabled(false);
     jbtPrev.setEnabled(false);
     jbtNext.setEnabled(false);
     jbtCancel.setText("Exit Simulation");
     isExitAllowed = true;
    }
   }
  }
  workSpacePanel.repaint();
 }
}
