 

import java.io.*;

/**
* Implements an object representing a FA state. An object of this type
* has two main fields which represent the FA state: the state's {@link
* #name}, and its {@link #acceptFlag accepting status or flag}. Two
* additional fields are provided to locate in a workspace the center of
* the circle representing the state in a FA transition diagram.
*/
public class State implements Serializable 
{
 //~ Instance variables ************************************************
 /** The state name. */
 private String name;

 /** The accepting status of the state. */
 private boolean acceptFlag;
 
 /**
 * The x-coordinate of the center of the circle representing the state
 * in a transition diagram.
 */
 private int x = 0;

 /**
 * The y-coordinate of the center of the circle representing the state
 * in a transition diagram.
 */
 private int y = 0;
 
 //~ Constructors ******************************************************
 /**
 * The default constructor for this class.
 */
 public State() 
 {
  this.name = "q0";
  this.acceptFlag = false;
 }

 /**
 * The constructor used where the state name is specified. This
 * method automatically sets the state's {@link #name name}.
 *
 * @param name the state name
 */
 public State(String name) 
 {
  this.name = name;
  this.acceptFlag = false;
 }

 /**
 * The constructor used where the state name and its accepting status
 * is specified. This method automatically sets the state's {@link
 * #name} and {@link #acceptFlag}.
 *
 * @param name the state name
 * @param accept the state's accepting status
 */
 public State(String name, boolean accept) 
 {
  this.name = name;
  this.acceptFlag = accept;
 }

 //~ Methods ***********************************************************
 /**
 * Sets the {@link #acceptFlag accepting status} of the state.
 *
 * @param accept the accepting status of the state
 */
 public void setAcceptFlag(boolean accept) 
 {
  this.acceptFlag = accept;
 }
 
 /**
 * Returns the accepting status of the state.
 *
 * @return {@link #acceptFlag this.acceptFlag}
 */
 public boolean getAcceptFlag() 
 {
  return this.acceptFlag;
 }

 /**
 * Sets the location of the center of the circle representing the
 * state in a transition diagram.
 *
 * @param a the {@link #x x-coordinate} of the state
 * @param b the {@link #y y-coordinate} of the state
 */
 public void setCoordinates(int a, int b) 
 {
  this.x = a;
  this.y = b;
 }

 /**
 * Sets the state's {@link #name}.
 *
 * @param name the state name
 */
 public void setName(String name) 
 {
  this.name = name;
 }

 /**
 * Returns the state's the {@link #name}.
 *
 * @return {@link #name this.name}
 */
 public String getName() 
 {
  return this.name;
 }

 /**
 * Returns the x-coordinate of the state's circle representation.
 *
 * @return {@link #x this.x}
 */
 public int getX() 
 {
  return this.x;
 }

 /**
 * Returns the y-coordinate of the state's circle representation.
 *
 * @return {@link #y this.y}
 */
 public int getY() 
 {
  return this.y;
 }
}
