 

import java.io.*;

/**
* Implements an object representing a FA transition. An object of this
* type has three main fields which represent the three elements of a FA
* transition function: the {@link #destination destination state}, the
* {@link #source source state}, and the {@link #input transition input}.
* All three of these must be specified in the FA; otherwise, the
* transition is invalid. A fourth field is added to {@link #index}
* transitions with similar transition paths.
*/

public class Transition implements Serializable 
{
 //~ Instance variables ************************************************
 /**
 * The destination state for the transition.
 *
 * @see State
 */
 private State destination = null;

 /**
 * The source state for the transition.
 *
 * @see State
 */
 private State source = null;

 /** The transition input. */
 private char input;

 /**
 * A counter for indexing the number of existing transitions on one
 * path alone.
 */
 private int index = 0;

 //~ Constructors ****************************************************** 
 /**
 * The default constructor for this class.
 */
 public Transition() 
 {

 }

 /**
 * The constructor which specifies the {@link #source} state, the
 * {@link #destination} state, the transition {@link #input}, and the
 * {@link #index} counter for the {@link Transition}. This
 * constructor sets all four fields of the transition to the given
 * paremeters.
 *
 * @param state1 the source state
 * @param state2 the destination state
 * @param c the transition input
 * @param n the index counter
 */
 public Transition(State state1, State state2, char c, int n) 
 {
  this.source = state1;
  this.destination = state2;
  this.input = c;
  this.index = n;
 }

 //~ Methods ***********************************************************
 /**
 * Sets the {@link #destination} state of the transition.
 *
 * @param state the {@link Transition} destination state
 */
 public void setDestination(State state) 
 {
  this.destination = state;
 }

 /**
 * Returns the destination state transition.
 *
 * @return {@link #destination this.destination}
 */
 public State getDestination() 
 {
  return this.destination; 
 }

 /**
 * Sets the transition's {@link #index} counter to compare this
 * transition with other transitions on the same path.
 *
 * @param n the index counter for this transition
 */
 public void setIndex(int n) 
 {
  this.index = n;
 }

 /**
 * Returns the transition index counter.
 *
 * @return {@link #index this.index}
 */
 public int getIndex() 
 {
  return this.index;
 }

 /**
 * Sets the transition input.
 *
 * @param input the transition input
 */
 public void setInput(char input) 
 {
  this.input = input;  
 }

 /**
 * Returns the transition input.
 *
 * @return {@link #input this.input}
 */
 public char getInput() 
 {
  return this.input; 
 }

 /**
 * Sets the {@link #source} state of the transition.
 *
 * @param state the transition source state
 */
 public void setSource(State state) 
 {
  this.source = state;
 }

 /**
 * Returns the source state of the transition.
 *
 * @return {@link #source this.source}
 */
 public State getSource() 
 {
  return this.source;
 }
}

